/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include "blade-config.h"
#include <iostream>
#include <sstream>
#include <QString>
#include <QUrl>

#include "CubeProxy.h"

#include "BladePlugin.h"
#include "PluginServices.h"
#include "BladeConstants.h"
#include "BladePluginWidget.h"

#include "SalsaViewerConnection.h"

using namespace blade;

cubepluginapi::PluginServices* blade_service;
/**
 * If the plugin doesn't load, start cube with -verbose to get detailed information.
 * If the error message "Plugin verification data mismatch" is printed, check if the same compiler
 * and Qt version have been used.
 */

#if QT_VERSION < 0x050000
Q_EXPORT_PLUGIN2( BladePlugin, BladePlugin );
#endif




/******************************************************************************************************
* start of CubePlugin implementation
******************************************************************************************************/

/** sets a version number, the plugin with the highest version number will be loaded */
void
BladePlugin::version( int& major, int& minor, int& bugfix ) const
{
    major  = 1;
    minor  = 0;
    bugfix = 0;
}

/**
 * returns the unique plugin name
 */
QString
BladePlugin::name() const
{
    return "Blade's OTF2 Trace Viewer";
}

QString
BladePlugin::getHelpText() const
{
    return "Simple OTF2 Trace viewer.";
}

/**
 * starting point of the plugin
 */
bool
BladePlugin::cubeOpened( cubepluginapi::PluginServices* service )
{
    active = true;
    blade_service = service;
    this->service = service;
    this->toolBar = new BladeToolBar( service );
    service->addToolBar( toolBar );
    toolBar->hide();
    widget_ = NULL;    
    QString cubepath = service->getCubeFileName();
	cubepath.remove("file://");
//    QUrl url = QUrl::fromLocalFile(cubepath );
    QFileInfo _fileinfo(cubepath );
    QString path = _fileinfo.absolutePath()+ "/traces.otf2";
 
    QFileInfo _trace( path );
    service->debug() <<  "Looking for trace " << cubepath << " __ " << path<< endl;;
     if (_trace.exists())
     {
        widget_ = new BladePluginWidget(path, service->getCube()->getCubeObject(), toolBar);
         
        // add tab to the system tab widget with the label this->label() and content this->widget()
        service->addTab( cubepluginapi::SYSTEM, this );
        
        setGlobalNotificationValue();
        
        checkForSalsaViewer();
        
        return true;
     }
     else
     {
	return false;
     }
    
     

//     return true; // initialisation is ok => plugin should be shown
}

void
BladePlugin::cubeClosed()
{
    delete widget_;
}

/******************************************************************************************************
* start of TabInterface implementation
******************************************************************************************************/

/**
 * returns the tab label of the created system tab
 */
QString
BladePlugin::label() const
{
    return "Blade";
}

/** returns the icon which will be placed left to the tab label */
QIcon
BladePlugin::icon() const
{
    return QIcon( ":/bladeicons/icon.png" );
}

/**
 * returns widget that will be placed into the tab
 */
QWidget*
BladePlugin::widget()
{
    return widget_;
}


/**
 * @brief BladePlugin::setActive is called when the tab gets activated or deactivated by selecting another tab
 * The tab related demo elements should only react on signals, if the tab is active. For that reason the
 * signals are disconnected if another tab becomes active.
 */
void
BladePlugin::setActive( bool _active )
{
    if ( _active )
    {
        active = true;
        toolBar->showSelf();
        connect( service, SIGNAL( treeItemIsSelected( cubepluginapi::TreeType, cubepluginapi::TreeItem* ) ),
                 this, SLOT( treeItemIsSelected( cubepluginapi::TreeType, cubepluginapi::TreeItem* ) ) );
        connect( service, SIGNAL( tabActivated( cubepluginapi::TreeType ) ),
                 this, SLOT( tabActivated( cubepluginapi::TreeType) ) );
        
        QString   txt = service->getSelection( cubepluginapi::METRICTREE )->getName();
        service->debug() << "setActive: last selected metric: " << txt << endl;
        txt = service->getSelection( cubepluginapi::CALLTREE )->getName();
        service->debug() << "setActive: last selected call path: " << txt << endl;
        
        valuesChanged();
        
    }
    else
    {
        toolBar->hideSelf();
        service->debug() << "setActive: deactivate " << endl;
        disconnect( service, SIGNAL( treeItemIsSelected( cubepluginapi::TreeType, cubepluginapi::TreeItem* ) ) );
        disconnect( service, SIGNAL( tabActivated( cubepluginapi::TreeType ) ) );;   
        active = false;
    }
}

/** This method is called, if the selected tree item has changed, the tree structure
 *  has changed or if an item is expanded or collapsed */
void
BladePlugin::valuesChanged()
{
    if (!active)
        return;
    cubegui::TreeItem* metric_item = service->getSelection( cubepluginapi::METRICTREE );
    cubegui::TreeItem* callpath_item = service->getSelection( cubepluginapi::CALLTREE );
    cubegui::TreeItem* region_item = service->getSelection( cubepluginapi::CALLFLAT );
    cubegui::TreeItem* sysrem_item = service->getSelection( cubepluginapi::SYSTEMTREE );
    
    service->debug() << "valuesChanged:selected metric: " << metric_item->getName() << endl;
    service->debug() << "valuesChanged:selected call path: " << callpath_item->getName()  << endl;
    service->debug() << "valuesChanged:selected region: " << region_item->getName() << endl;
    service->debug() << "valuesChanged:selected sysres: " << sysrem_item->getName()  << endl;
    
//     if ( callpath_item->isAggregatedLoopItem() )
//     {
//         QList<cube::Cnode*> iterations = ( static_cast<AggregatedTreeItem*> ( callpath_item ) )->getIterations();
//         int                 min        = std::min( 10, iterations.size() );
//         if ( min > 0 )
//         {
//             service->debug() << "selected loop item: cnode_ids of first 10 iterations:" << endl;
//         }
//         for ( int i = 0; i < min; i++ )
//         {
//             cube::Cnode*        cnode = iterations.at( i );
//             service->debug() << cnode->get_id() << endl;
//         }
//     }
//     if ( callpath_item->getCubeObject() == 0 )
//     {
//         service->debug() << "selected aggregated item" << callpath_item->getName() << endl;
//     }
//     else
//     {
//         cube::Cnode*                                    cnode = static_cast<cube::Cnode*> ( callpath_item->getCubeObject() );
//         ((BladeWidget*)widget_)->selectedCallPath(cnode, callpath_item->isExpanded() );
//     }
//     
    cube::Cnode*                                     cnode = static_cast<cube::Cnode*> ( callpath_item->getCubeObject() );
    cube::Metric*                                    metric = static_cast<cube::Metric*> ( metric_item->getCubeObject() );
    cube::Sysres*                                    sysres = static_cast<cube::Sysres*> ( sysrem_item->getCubeObject() );
    cube::Region*                                    region = static_cast<cube::Region*> ( region_item->getCubeObject() );
    ((BladePluginWidget*)widget_)->selectionChanged(metric, metric_item->isExpanded(), 
                                              cnode, callpath_item->isExpanded(),
                                              sysres, sysrem_item->isExpanded(),
                                              region, region_item->isExpanded() );
    
    
}


/** Returns the minimal width that is necessary to display all informations.
    This value is used in the menu item "Display->Optimize width".
 */
QSize
BladePlugin::sizeHint() const
{
    return QSize( 100, 100 );
}

// ******************************************************************************************************


/******************************************************************************************************
* implementation of slots to handle signals of PluginServices
******************************************************************************************************/

/**
 * @brief BladePlugin::loadSettings loads settings from global setting file
 * the plugin should create a group with its name as group label
 */
void
BladePlugin::loadGlobalSettings( QSettings& settings )
{
    int numCalls = settings.value( "BladePluginCalls", 0 ).toInt();

    service->debug() << "loadSettings: " << numCalls << endl;
}

/**
   Saves settings system specific file, for UNIX systems to $HOME/.config/FZJ/Cube.conf
 */
void
BladePlugin::saveGlobalSettings( QSettings& settings )
{
    int numCalls = settings.value( "BladePluginCalls", 0 ).toInt(); // 0 is default, if no value exists
    settings.setValue( "BladePluginCalls", ++numCalls );
}

QString
BladePlugin::settingName()
{
    return "bladePlugin";
}

void
BladePlugin::treeItemIsSelected( cubepluginapi::TreeType type, cubegui::TreeItem* item )
{
    valuesChanged();
}

void
BladePlugin::tabActivated( cubepluginapi::TreeType typ )
{
    ((BladePluginWidget*)widget_)->tabActivated( typ );
    valuesChanged();
}


/**
 * called, if the user has selected the menu item "Display->Dimension Order"
 */
void
BladePlugin::orderHasChanged( const QList<cubepluginapi::DisplayType>& order )
{
    service->debug() << "orderHasChanged user has changed the dimension order" << endl;

    // example: disable demo plugin, if system widget is first
    bool enabled = order.at( 0 ) != cubepluginapi::SYSTEM;
    service->enableTab( this, enabled );
}

void
BladePlugin::checkLoop( cubepluginapi::UserAction type )
{
    if ( type == cubepluginapi::LoopMarkAction )
    {
        service->debug() << "user has marked tree item as loop" << endl;
    }
}


/**
 * @brief DemoPlugin::globalValueChanged is called if the value of a global variable has changed. In this
 * example, the value of "testVal" is incremented if the plugin menu item is selected.
 */
void
BladePlugin::globalValueChanged( const QString& name )
{
    service->getGlobalValue( salsaviewer::SALSAVIEWER_PLUGIN_NOTIFIER ).toString();
}


/**
 * slot, which is called if "Demo Menu Item" is selected
 */
void
BladePlugin::setGlobalNotificationValue()
{
    int val = 1;
    service->setGlobalValue( blade::BLADE_PLUGIN_NOTIFIER , QVariant( val ), false ); // true = notify myself
}


/**
 * slot, which is called if "Demo Menu Item" is selected
 */
bool
BladePlugin::checkForSalsaViewer()
{
//     service->setGlobalValue( BladePlugin::BLADE_PLUGIN_NOTIFIER , QVariant( ++val ), false ); // true = notify myself
}

