/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include "blade-config.h"
#include <algorithm> 
#include <iomanip>
#include <QLabel>
#include <QGraphicsView>
#include <QGraphicsScene>
#include <QRectF>
#include <QVBoxLayout>
#include <QScrollArea>
#include <QAction>
#include <QComboBox>

#include "Service.h"
#include "BladeWidget.h"
#include "SubTraceWidget.h"
#include "MetricTraceWidget.h"
#include "SubTraceWidgetHolder.h"
#include "TraceControl.h"

#include "TraceEventGroupWidget.h"
TraceEventGroupWidget * tegw= NULL;
Trace * trace = NULL;


TraceEventFilter * cf_filter = NULL;

SelectionFilter * s_filter = NULL;
ZoomFilter * z_filter = NULL;
RoleFilter * role_filter = NULL;
RegionPropertiesFilter * rp_filter = NULL;

BladeWidget::BladeWidget(Trace * _trace, cube::Cube*_cube, QWidget * parent):QWidget(parent),cube(_cube)
{
    compact = false;
    trace = _trace; // initialization of a global trace

    no_callpath = trace->is_selection_filter_active();
    info_widget = new TraceInfoWidget( trace );
    c_filter = new CallPathFilter();
    c_filter->set_active( trace->is_callpath_filter_active());
    c_filter->set_call_path( 0, false );
    
    f_filter = new FlatFilter();
    f_filter->set_active(false);
    
    cf_filter = c_filter;
    
    
    s_filter = new SelectionFilter(cube);
    s_filter->set_active( trace->is_selection_filter_active());
    
    
    z_filter = new ZoomFilter();
    z_filter->set_zoom_window( trace->get_start_timestamp() , trace->get_end_timestamp());
    
    role_filter = new RoleFilter();
    rp_filter = new RegionPropertiesFilter();
    
    
    QVBoxLayout* layout = new QVBoxLayout();
    
    z_overview_filter = new ZoomFilter();
    z_overview_filter->set_zoom_window( trace->get_start_timestamp() , trace->get_end_timestamp());
    

    QHBoxLayout* h_layout = new QHBoxLayout();
    h_layout->setAlignment ( Qt::AlignVCenter );
    layout->addLayout( h_layout );
    overview = new  OverviewTraceWidget(0, trace, &cf_filter, s_filter, z_filter, z_overview_filter);
    overview->setSizePolicy(QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed) );

    h_layout->addWidget( overview );
    overview_selector = new QComboBox();
    h_layout->addWidget( overview_selector );
    std::map<uint64_t, LocationClass>& _locs = trace->get_locations();
    std::vector<uint64_t> _sorted_locations;
    for (std::map<uint64_t, LocationClass>::iterator iter = _locs.begin(); iter!=_locs.end(); ++iter)
    {
	_sorted_locations.push_back((*iter).first); 
    }
    std::sort(_sorted_locations.begin(), _sorted_locations.end(), bladeservices::compare_locations<uint64_t>);
    for ( std::vector<uint64_t>::iterator iter = _sorted_locations.begin(); iter!=_sorted_locations.end(); ++iter)
    {
	const QVariant _index = QVariant( qulonglong((*iter)) );
	overview_selector->addItem(QString::fromStdString( _locs[(*iter) ].name), _index ); 
    }
    connect( overview_selector, SIGNAL(currentIndexChanged ( int  )), this, SLOT(updateOverview(int))); 
    
    
    
    QScrollArea * _scroll_container = new QScrollArea();
    _scroll_container->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding) );
    _scroll_container->setWidgetResizable(true);
    _scroll_container->setVerticalScrollBarPolicy( Qt::ScrollBarAsNeeded );
    layout->addWidget( _scroll_container );
    QWidget * _grid_widget = new QWidget();
    QGridLayout* _layout = new QGridLayout();
    _grid_widget->setLayout(_layout);
    _layout->setSpacing ( 0 );
    _layout->setContentsMargins (0, 0, 0,0 );
    _layout->setSizeConstraint( QLayout::SetMinimumSize );
    _scroll_container->setWidget( _grid_widget  );      
    
    std::map<uint64_t, TraceEvent*>::iterator loc_iter= trace->get_location_entries().begin();
    for ( ; loc_iter!= trace->get_location_entries().end(); ++loc_iter )
    {
	SubTraceWidgetHolder _stwh;
	_stwh.location = loc_iter->first;
	_stwh.widget =  new SubTraceWidget(loc_iter->first);
	_stwh.subtrace_label =  new QLabel( trace->get_location(_stwh.location).name.c_str());
	subtraces.push_back( _stwh );
	connect( _stwh.widget, SIGNAL(cursorMoved(QPointF)), this, SLOT(updateCursors(QPointF)));          

    }
    std::stable_sort(subtraces.begin(), subtraces.end(), bladeservices::compare_locations<SubTraceWidgetHolder> );
    int i=0;
    std::map<uint64_t, TraceControl*> _trace_controls;
    std::map<uint64_t, std::vector<QWidget*> > _widgets_to_hide;
    
    for ( std::vector<SubTraceWidgetHolder>::iterator iter = subtraces.begin(); iter!= subtraces.end(); ++iter )
    {
        _layout->setRowMinimumHeight ( i, 0 );
	_layout->addWidget( (*iter).subtrace_label , i, 0, Qt::AlignVCenter);
	_layout->addWidget( (*iter).widget, i, 1, Qt::AlignVCenter);
	
	QLabel * c_label = new QLabel(  "Counters ");
	MetricTraceWidget * _mtw = (*iter).widget->getMetricTraceWidget() ;
	QToolBar * add_metric = new QToolBar( );
	QAction * _add_metric_action = add_metric->addAction(QIcon(":/bladeicons/modify_counterline.png"), "modify couterline");
	if (_mtw != NULL)
	{
	    connect( _mtw, SIGNAL(cursorMoved(QPointF)), this, SLOT(updateCursors(QPointF)));             
	    connect( _add_metric_action, SIGNAL( triggered( void )), _mtw, SLOT(addCounterLine()));
	}
	if (bladeservices::thread_id((*iter).location) == 0)
	{  
	    _layout->addWidget(c_label, ++i, 0, Qt::AlignVCenter);
	    c_label->hide();
	    _layout->addWidget(_mtw, i , 1, Qt::AlignVCenter);
	    _mtw->hide();   
	    _layout->addWidget(add_metric, i, 2, Qt::AlignTop);
	    add_metric->hide();
	}
	if (bladeservices::thread_id((*iter).location) == 0)
	{
	    TraceControl * _tc = new TraceControl();
	    (*iter).trace_control = _tc;
	    _layout->addWidget(_tc, i-1, 2, Qt::AlignVCenter); 
	    _trace_controls[bladeservices::master_id((*iter).location)] = _tc;
	    _tc->addMetricsToHide( c_label );
	    _tc->addMetricsToHide( _mtw ); 
	    _tc->addMetricsToHide( add_metric );
	}else
	{
	    TraceControl * _tc = _trace_controls[ bladeservices::master_id((*iter).location)];
	    (*iter).trace_control = _tc;
	    _tc->addWidgetToHide((*iter).subtrace_label );
	    _tc->addWidgetToHide((*iter).widget );
	    (*iter).subtrace_label->hide();
	    (*iter).widget->hide();
	}
	i++;
    }
    this->setLayout( layout );
    
    
    
    tegw = new TraceEventGroupWidget( );
	
	
}
    
    
   
BladeWidget::~BladeWidget()
{
        delete c_filter;
        delete z_filter;
        delete f_filter;
        delete s_filter;
        delete role_filter;
        delete rp_filter;
        delete z_overview_filter;
	delete tegw;
	delete trace;
}
    

void
BladeWidget::compactToggle()
{
    compact = !compact;
    for ( std::vector<SubTraceWidgetHolder>::iterator iter = subtraces.begin(); iter!= subtraces.end(); ++iter )
    {
	(*iter).widget->compactToggle( compact );
	if (compact)
	{
 	    if (!((*iter).trace_control->isHidden()))
		(*iter).trace_control->toggleHiding();
	    (*iter).widget->show();
	    (*iter).trace_control->hide();
	    (*iter).subtrace_label->hide();
	}
	else
	{
	    if (bladeservices::thread_id((*iter).location) == 0)
	    {
		(*iter).trace_control->toggleHiding();
	    }
	    (*iter).trace_control->show();
	    (*iter).subtrace_label->show();
	}
	
	
	
	
    }
}
      
    
    
void
BladeWidget::info()
{
    info_widget->show();

}
    
       
void
BladeWidget::unZoom()
{
    z_filter->unzoom();
}
   
void
BladeWidget::goBack()
{
    z_filter->go_back();
}   
   
   
   
void
BladeWidget::moveLeft()
{
    z_filter->move_left(); 
}

void
BladeWidget::moveRight()
{
     z_filter->move_right();
}
    
    
void
BladeWidget::updateCursors(QPointF _p)
{
    for ( std::vector<SubTraceWidgetHolder>::iterator iter = subtraces.begin(); iter!= subtraces.end(); ++iter )
    {
        (*iter).widget->setCursor( _p );
    }
}
    
    
        
void
BladeWidget::updateOverview( int _index)
{
    QVariant variant = overview_selector->itemData ( _index );
    overview->setLocation(variant.toULongLong());
}
       
void
BladeWidget::selectionChanged( cube::Metric* _m, bool _m_is_expanded,
                               cube::Cnode* _cp, bool _c_is_expanded,
                               cube::Sysres * _s, bool _s_is_expanded,
                               cube::Region * _r, bool _r_is_expanded)
{
    c_filter->set_call_path( _cp->get_id(), _c_is_expanded );
    if (_r!=NULL)
        f_filter->set_region(_r->get_id(), _c_is_expanded);
    
    s_filter->set_metric( _m, (_m_is_expanded)?cube::CUBE_CALCULATE_EXCLUSIVE:cube::CUBE_CALCULATE_INCLUSIVE);
    
    overview->updateScene(true);
    for ( std::vector<SubTraceWidgetHolder>::iterator iter = subtraces.begin(); iter!= subtraces.end(); ++iter )
    {
        (*iter).widget->updateScene();
    }
}
   
void
BladeWidget::tabActivated( cubepluginapi::TreeType typ )
{
    if (!no_callpath)
    {
        if (typ == cubepluginapi::CALLFLAT)
        {
            f_filter->set_active(true);
            c_filter->set_active(false);
            cf_filter = f_filter;
        }  if (typ == cubepluginapi::CALLTREE)
        {
            f_filter->set_active(false);
            c_filter->set_active(true);
            cf_filter = c_filter;
        }
    }
}
  
  
