/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include "blade-config.h"
#include <iostream>
#include <cstdlib>
#include <algorithm> 
#include <QMouseEvent>
#include <QToolTip>
#include "GraphicsView.h"
#include "BladeUtils.h"
#include "TraceEventRect.h"
#include "TraceEventGroupWidget.h"
#include "Service.h"
#include "TraceEventGroupWidget.h"


extern TraceEventGroupWidget * tegw;

GraphicsView::GraphicsView ( QGraphicsScene* scene) : QGraphicsView( scene )
{
    zooming = false;
    pressed = false;
    mouse_hover = false;
    x_treshold = 5;
    setMouseTracking(true);
    start_position = QPoint(0, 0);
    end_position = QPoint(0, 0);
    
    zoom_rect = calculateRect( start_position, end_position );
    zoom_window = calculateZoomF();
    setContentsMargins (0, 0, 0,0 );
}; 
    
   
void
GraphicsView::updateScene()
{
    zoom =  new  QGraphicsRectItem();
    cursor = new QGraphicsLineItem();
    
    QPen pen;
    pen.setColor( QColor( 0,0,0,100));
    QBrush brush;
    brush.setColor( QColor( 0,0,0,100));
    brush.setStyle(Qt::SolidPattern);
    
    
    QPen cursor_pen;
    cursor_pen.setColor( QColor( 0,0,0,255));
    cursor_pen.setWidth(2);
    cursor->setPen( cursor_pen );
    
    
    zoom->setPen( pen );
    zoom->setBrush( brush );
    
    zoom->setRect( zoom_rect );
    
    scene()->addItem( zoom );
    scene()->addItem( cursor );
}
    
QPointF&
GraphicsView::getZoomF()
{
    return zoom_window;
}
    
QPointF
GraphicsView::getCursorPosition()
{
    return QPointF(
        (double)end_position.x()/ (double)width()
        ,0
    );
}
  
void
GraphicsView::setCursorPosition(QPointF& _p)
{
    cursorPosition = QPoint( _p.x() * width(),0) ;
    cursor->setLine(cursorPosition.x(),0, cursorPosition.x(), height() );
}  
  
  
QRect 
GraphicsView::calculateRect( const QPoint& _s, const QPoint& _f)
{
    QSize _size = QSize(abs( _s.x() - _f.x()), height());
    
    QPoint _p = QPoint(std::min( _s.x(), _f.x()), 0 ) ;
    QRect _r = QRect( _p, _size);
    return _r;
}
    
    
    
QPointF
GraphicsView::calculateZoomF( )
{
    QPointF _p = QPointF(
        (double)zoom_rect.x()/ (double)width()
        ,
        (double)(zoom_rect.x() + zoom_rect.width())/ (double)width()
    );

    return _p;
}
     
    
void 
GraphicsView::mousePressEvent(QMouseEvent * _event)
{
    if (_event->button() == Qt::RightButton)
        return;
    int _x = _event->x();
    int _y = _event->y();
    if ( _y < 0 || _y > height())
        return;
    if ( _x < 0 || _x > width())
        return;
    
    zooming = false;
    pressed = true;
    start_position.setX( _x );
    start_position.setY( _y );
    end_position.setX( _x );
    end_position.setY( _y );
    zoom_rect = calculateRect( start_position, end_position );
    zoom->setRect(zoom_rect);
}

void
GraphicsView::mouseMoveEvent(QMouseEvent * _event)
{

    int _x = _event->x();
    int _y = _event->y();
    if ( _y < 0 || _y > height())
        return;
    
    if ( _x < 0 || _x > width())
    {
        mouseReleaseEvent( _event);
        return;
    }
    
    if (!zooming && pressed)
        if ( std::abs(_x - start_position.x()) > x_treshold)
            zooming = true;
        else
            return;
   if (mouse_hover)
	return;

   mouse_hover = true; 
    
    end_position.setX( _x );
    end_position.setY( _y );     
    

    if (!zooming)
    {
        TraceEventRect * _rect = dynamic_cast<TraceEventRect*>( itemAt(_x, _y));
        if (_rect != NULL )
        {
            QString _tip = 
            "Region name: " +
            BladeUtils::shortCutRegionName(_rect->getRegionName())+"\n"
            "Start time : "+
            _rect->getStartTime() + "\n"
            "Duration : " +
            _rect->getDuration() + "\n"
            "Call Path: \n" +
            _rect->getCallPath() + "\n"
            "Metrics: \n" + 
             _rect->getMetrics();
                
            QToolTip::showText(mapToGlobal(QPoint(_x, _y)), _tip );     
        }else
        {
            QToolTip::hideText();
        }
        mouse_hover=false;
        emit cursorMoved(getCursorPosition());
        return;
    }
    else
    {       
        mouse_hover=false;
        zoom_rect = calculateRect( start_position, end_position );
        zoom_window = calculateZoomF();
        zoom->setRect(zoom_rect);
        emit zoomingSelection();        
    }
}

void 
GraphicsView::mouseReleaseEvent(QMouseEvent * _event)
{
    int _x = _event->x();
    int _y = _event->y();
    if (!zooming)
    {
        if (pressed)
        {
             TraceEventRect * _rect = dynamic_cast<TraceEventRect*>( itemAt(_x, _y));
            if (_rect != NULL )
            {
                QString _tip = 
                "Region name: " +
            BladeUtils::shortCutRegionName(_rect->getRegionName())+"\n"
                "Start time : "+
                _rect->getStartTime() + "\n"
                "Duration : " +
                _rect->getDuration() + "\n"
                "Call Path: \n" +
                _rect->getCallPath();
                    
                QToolTip::showText(mapToGlobal(QPoint(_x, _y)), _tip );   
                
                TraceEventGroup * _teg =  _rect->getTraceEvent()->get_events_group();
                std::vector<TraceEvent*> _events;
                if (_teg == NULL)
                    _events.push_back( _rect->getTraceEvent());
                else
                    _events= _teg->get_events();
                std::sort(_events.begin(), _events.end(), bladeservices::compare_locations<const TraceEvent*>);
                
                tegw->displayEvents( _events, _teg );
                
                
            }
        }
        start_position.setX( 0 );
        start_position.setY( 0 );
        end_position.setX( 0 );
        end_position.setY( 0 );
        pressed = false;
        return;  
    }
    pressed = false;
    if ( _x < 0 )
    {
        _x =0 ;
    }  
    if (  _x > width())
    {
        _x = width() ;
    }  
    end_position.setX( _x );
    end_position.setY( _y );     
    zoom_rect = calculateRect( start_position, end_position );   
    zoom_window = calculateZoomF();
    
    zooming = false;
    start_position.setX( 0 );
    start_position.setY( 0 );
    end_position.setX( 0 );
    end_position.setY( 0 );
    zoom_rect = calculateRect( start_position, end_position );
    zoom->setRect(zoom_rect);
    
    emit zoomChanged();
    
}


     
void
GraphicsView::showToolTip( QString _tip)
{
    QToolTip::showText(mapToGlobal(end_position),_tip );  
}
