/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/



#include "blade-config.h"
#include "TraceEventGroupEarlyReduceAnalyzer.h"
#include "Service.h"
#include "Trace.h"
#include "ReduceTraceEvent.h"

extern Trace* trace;

TraceEventGroupEarlyReduceAnalyzer::TraceEventGroupEarlyReduceAnalyzer(): TraceEventGroupAnalyzer()
{

    name = "Early Reduce";
    description = "Collective communication\n"
                  "operations that send data\n"
                  "from all processes to one\n"
                  "destination process (i.e., n-to-1)\n"
                  "may suffer from waiting times\n"
                  "if the destination process enters\n"
                  "the operation earlier than its\n"
                  "sending counterparts, that is,\n"
                  "before any data could have\n"
                  "been sent. The pattern refers to\n"
                  "the time lost as a result of\n"
                  "this situation.";
    
}

TraceEventGroupEarlyReduceAnalyzer::~TraceEventGroupEarlyReduceAnalyzer()
{
}


bool
TraceEventGroupEarlyReduceAnalyzer::probe(std::vector<TraceEvent*>& events)
{
    bool only_reduce = true;
    bool active = true;
    for (std::vector<TraceEvent*>::const_iterator iter = events.begin(); iter != events.end(); ++iter)
    {
        TraceEvent * _e = *iter;
        only_reduce &= (_e->get_region_role( ) == OTF2_REGION_ROLE_COLL_ALL2ONE );
        ReduceTraceEvent * _as_bcast = dynamic_cast<ReduceTraceEvent*>(_e);
        only_reduce &= (_as_bcast !=  NULL);
        if (_e->is_group_leader())
        {
            root_t = _e->get_start_timestamp();
        }
    }
    for (std::vector<TraceEvent*>::const_iterator iter = events.begin(); iter != events.end(); ++iter)
    {
        TraceEvent * _e = *iter;
        if (!_e->is_group_leader())
        {
            max_next = std::min( max_next, (*iter)->get_start_timestamp() );
        }
    }

    if (max_next<= root_t)
        active = false;
    
    if (!only_reduce || !active)
        return false;

    return true;
}

void
TraceEventGroupEarlyReduceAnalyzer::analyze(std::vector<TraceEvent*>& events, int dh, int boxwidth )
{
    TraceEventGroupAnalyzer::analyze( events, dh, boxwidth  );
    clear_analyzer();

    int i=0;
    for (std::vector<TraceEvent*>::iterator iter = events.begin(); iter != events.end(); ++iter, ++i)
    {
        if ((*iter)->is_group_leader())
        {
            std::pair<double,double> _dp;
            _dp.first =  ((double)(root_t - group_min))/(double)(group_max - group_min);
            _dp.second = ((double)(max_next - group_min ))/(double)(group_max - group_min);
            
            TraceEventRect * _rect = new TraceEventRect(root_t -(*iter)->get_start_timestamp(),   trace, (*iter), QRect(_dp.first * boxwidth, location_line_index[i]*dh, ( _dp.second - _dp.first)*boxwidth, dh) );

            QGraphicsSimpleTextItem * _dur_label = new QGraphicsSimpleTextItem( _rect->getDuration() );
            _dur_label->setPos(boxwidth+10,  location_line_index[i]*dh);
            dynamic_labels.push_back(_dur_label); 
  
            _rect->setPen( QColor(127, 0, 0, 200) );
            QBrush brush;
            brush.setColor( QColor(127, 0, 0, 200)  );
            brush.setStyle( Qt::SolidPattern );
            _rect->setBrush(brush );
                    
            rects.push_back( _rect );
        }
    }
}


