/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/



#include "blade-config.h"
#include "TraceEventGroupLateBroadcastAnalyzer.h"
#include "Service.h"
#include "Trace.h"
#include "BcastTraceEvent.h"

extern Trace* trace;

TraceEventGroupLateBroadcastAnalyzer::TraceEventGroupLateBroadcastAnalyzer(): TraceEventGroupAnalyzer()
{

    name = "Late Broadcast";
    description = "Collective communication operations\n"
                  "that send data from one source process\n"
                  "to all processes (i.e., 1-to-n) may\n"
                  "suffer from waiting times if destination\n"
                  "processes enter the operation earlier\n"
                  "than the source process, that is,\n"
                  "before any data could have been sent.";
    
}

TraceEventGroupLateBroadcastAnalyzer::~TraceEventGroupLateBroadcastAnalyzer()
{
}


bool
TraceEventGroupLateBroadcastAnalyzer::probe(std::vector<TraceEvent*>& events)
{
    bool only_bcast = true;
    bool active = false;
    for (std::vector<TraceEvent*>::const_iterator iter = events.begin(); iter != events.end(); ++iter)
    {
        TraceEvent * _e = *iter;
        only_bcast &= (_e->get_region_role( ) == OTF2_REGION_ROLE_COLL_ONE2ALL );
        BcastTraceEvent * _as_bcast = dynamic_cast<BcastTraceEvent*>(_e);
        only_bcast &= (_as_bcast !=  NULL);
        if (_e->is_group_leader())
            root_t = _e->get_start_timestamp();
    }
    for (std::vector<TraceEvent*>::const_iterator iter = events.begin(); iter != events.end(); ++iter)
    {
        TraceEvent * _e = *iter;
        if (_e->get_start_timestamp()< root_t)
            active = true;
    }

    if (!only_bcast || !active)
        return false;

    return true;
}

void
TraceEventGroupLateBroadcastAnalyzer::analyze(std::vector<TraceEvent*>& events, int dh, int boxwidth )
{
    TraceEventGroupAnalyzer::analyze( events, dh, boxwidth  );
    clear_analyzer();

    int i=0;
    for (std::vector<TraceEvent*>::iterator iter = events.begin(); iter != events.end(); ++iter, ++i)
    {
        if (!(*iter)->is_group_leader() && ( (*iter)->get_start_timestamp() < root_t))
        {
            std::pair<double,double> _dp;
            _dp.first =  ((double)((*iter)->get_start_timestamp() - group_min))/(double)(group_max - group_min);
            _dp.second = ((double)(root_t - group_min ))/(double)(group_max - group_min);
            
            TraceEventRect * _rect = new TraceEventRect(root_t -(*iter)->get_start_timestamp(),   trace, (*iter), QRect(_dp.first * boxwidth, location_line_index[i]*dh, ( _dp.second - _dp.first)*boxwidth, dh) );

            QGraphicsSimpleTextItem * _dur_label = new QGraphicsSimpleTextItem( _rect->getDuration() );
            _dur_label->setPos(boxwidth+10,  location_line_index[i]*dh);
            dynamic_labels.push_back(_dur_label); 
  
            _rect->setPen( QColor(127, 0, 0, 200) );
            QBrush brush;
            brush.setColor( QColor(127, 0, 0, 200)  );
            brush.setStyle( Qt::SolidPattern );
            _rect->setBrush(brush );
                    
            rects.push_back( _rect );
        }
    }
}


