/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/

#include "jengafett-config.h"
#include <QVBoxLayout>
#include <QtPlugin>

#include <QGraphicsView>
#include <QGraphicsScene>
#include <QPointF>
#include <QVector>

#include <algorithm>
#include <limits>
#include <iostream>
#include "JengaFettPlugin.h"
#include "PluginServices.h"


#include "CubeCnode.h"

using namespace std;
using namespace jengafett_plugin::gui;

 cubepluginapi::PluginServices* jengafett_service;
/**
 * If the plugin doesn't load, start cube with -verbose to get detailed information.
 * If the error message "Plugin verification data mismatch" is printed, check if the same compiler
 * and Qt version have been used.
 */

#if QT_VERSION < 0x050000
Q_EXPORT_PLUGIN2( JengaFettPlugin, JengaFettPlugin );
#endif

/******************************************************************************************************
* start of CubePlugin implementation
******************************************************************************************************/

/** sets a version number, the plugin with the highest version number will be loaded */
void
JengaFettPlugin::version( int& major, int& minor, int& bugfix ) const
{
    major  = 1;
    minor  = 0;
    bugfix = 0;
}

/**
 * returns the unique plugin name
 */
QString
JengaFettPlugin::name() const
{
    return "Jenga Fett Plugin";
}

QString
JengaFettPlugin::getHelpText() const
{
    return "Displays values of hierarhical metrics as a stacked bars (Jenga). ";
}

/**
 * starting point of the plugin
 */
bool
JengaFettPlugin::cubeOpened( cubepluginapi::PluginServices* service )
{
    this->service = service;
    jengafett_service = service;
    this->toolBar = new JengaFettToolBar( service );
    service->addToolBar( toolBar );
    toolBar->hide();
    widget_ = new JengaFettPluginWidget(service->getCube(), toolBar);

    
    // add tab to the system tab widget with the label this->label() and content this->widget()
    service->addTab( cubepluginapi::SYSTEM, this );
    
        connect( service, SIGNAL( contextMenuIsShown( cubepluginapi::TreeType, cubepluginapi::TreeItem* ) ),
            this, SLOT( contextMenuIsShown( cubepluginapi::TreeType, cubepluginapi::TreeItem* ) ) );

    return true; // initialisation is ok => plugin should be shown
}

void
JengaFettPlugin::cubeClosed()
{
    delete widget_;
}


/******************************************************************************************************
* start of TabInterface implementation
******************************************************************************************************/

/**
 * returns the tab label of the created system tab
 */
QString
JengaFettPlugin::label() const
{
    return "Jenga Fett";
}

/** returns the icon which will be placed left to the tab label */
QIcon
JengaFettPlugin::icon() const
{
    return QIcon( ":/jengafetticons/jengafetticon.png" );
}

/**
 * returns widget that will be placed into the tab
 */
QWidget*
JengaFettPlugin::widget()
{
    return widget_;
}


/** This method is called, if the selected tree item has changed, the tree structure
 *  has changed or if an item is expanded or collapsed */
void
JengaFettPlugin::valuesChanged()
{
/** Returns selected items in the given tree  @param type METRICTREE, CALLTREE, CALLFLAT, SYSTEMTREE */
    const QList<cubepluginapi::TreeItem*>& metrics = service->getSelections( cubepluginapi::METRICTREE );
    const QList<cubepluginapi::TreeItem*>& cnodes = service->getSelections( cubepluginapi::CALLTREE );
    const QList<cubepluginapi::TreeItem*>& sysres = service->getSelections( cubepluginapi::SYSTEMTREE );
    
    widget_->selectedItems( metrics, cnodes, sysres );
}

/**
 * @brief DemoPlugin::setActive is called when the tab gets activated or deactivated by selecting another tab
 * The tab related demo elements should only react on signals, if the tab is active. For that reason the
 * signals are disconnected if another tab becomes active.
 */
void
JengaFettPlugin::setActive( bool active )
{
    if ( active )
    {
        toolBar->showSelf();
        connect( service, SIGNAL( treeItemIsSelected( cubepluginapi::TreeType, cubepluginapi::TreeItem* ) ),
                 this, SLOT( treeItemIsSelected( cubepluginapi::TreeType, cubepluginapi::TreeItem* ) ) );
    }
    else
    {
        toolBar->hideSelf();
        service->disconnect( service, SIGNAL( treeItemIsSelected( cubepluginapi::TreeType, cubepluginapi::TreeItem* ) ) );
    }
}


/** Returns the minimal width that is necessary to display all informations.
    This value is used in the menu item "Display->Optimize width".
 */
QSize
JengaFettPlugin::sizeHint() const
{
    return QSize( 100, 100 );
}



void
JengaFettPlugin::treeItemIsSelected( cubepluginapi::TreeType, cubepluginapi::TreeItem* item )
{
    (void) item;
    valuesChanged();
}


void
JengaFettPlugin::contextMenuIsShown( cubepluginapi::TreeType type, cubepluginapi::TreeItem* item )
{
     
    if (type != cubepluginapi::METRICTREE)
        return;
            
    contextItem     = item;
    contextTreeType = type;

    QAction*    includeIntoGraphGroup = service->addContextMenuItem( type, "Include Metric to Jenga Plot " );
    connect( includeIntoGraphGroup, SIGNAL( triggered() ), this, SLOT( inclMetricToGraph() ) );
    
    QAction*    clearJengaPlot = service->addContextMenuItem( type, "Clear Jenga Plot " );
    connect( clearJengaPlot, SIGNAL( triggered() ), this, SLOT( clearJengaPlot() ) );
}


void
JengaFettPlugin::inclMetricToGraph( )
{
    widget_->addMetricToGraph( contextItem->getCubeObject() );
    valuesChanged();
}

void
JengaFettPlugin::clearJengaPlot( )
{
    widget_->clearJengaPlot( );
    valuesChanged();
}
