/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2019                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include <iostream>
#include <sstream>
#include <QtPlugin>

#include "CubeProxy.h"

#include "ScorePionPlugin.h"
#include "ScorePionToolBar.h"
#include "PluginServices.h"
#include "ScorePionConfigFile.h"
#include "ScorePionFiltersFactory.h"

using namespace scorepion_plugin::gui;
using namespace scorepion_plugin::misc;
using namespace scorepion_plugin::filters;

 cubepluginapi::PluginServices* scorepion_service;
/**
 * If the plugin doesn't load, start cube with -verbose to get detailed information.
 * If the error message "Plugin verification data mismatch" is printed, check if the same compiler
 * and Qt version have been used.
 */

#if QT_VERSION < 0x050000
Q_EXPORT_PLUGIN2( ScorePionPlugin, ScorePionPlugin );
#endif

/******************************************************************************************************
* start of CubePlugin implementation
******************************************************************************************************/

/** sets a version number, the plugin with the highest version number will be loaded */
void
ScorePionPlugin::version( int& major, int& minor, int& bugfix ) const
{
    major  = 1;
    minor  = 0;
    bugfix = 0;
}

/**
 * returns the unique plugin name
 */
QString
ScorePionPlugin::name() const
{
    return "ScorePion";
}

QString
ScorePionPlugin::getHelpText() const
{
    return "Simple Score-P's filter file creator.";
}

/**
 * starting point of the plugin
 */
bool
ScorePionPlugin::cubeOpened(  cubepluginapi::PluginServices* service )
{
    widget_ = NULL;
    active = true;
    scorepion_service = service;
    this->service = service;

    if (service->getCube()->getMetric("time") == NULL)
        return false;


    QString cubepath = service->getCubeFileName();
    QFileInfo _fileinfo( cubepath);
    QString path = _fileinfo.absolutePath()+ "/scorep.cfg";
 
    QFileInfo configfile( path );
     
    filter_factory = new ScorePionFiltersFactory();
    
     if (configfile.exists())
     {
        config_file = new ScorePionConfigFile( path  );
            
        QString filter = config_file->getFilterFile();
        QString filter_path = _fileinfo.absolutePath()+ "/" + filter;

        QFileInfo filterfile( filter_path );
            
        if (filterfile.exists())
        {
            std::cout << " file exists " << filter_path.toUtf8().data() << std::endl;
            filter_factory->readFilterFile(filter_path);
        } 
        else
        {
            std::cout << "no filter file " << std::endl;   
        }
    }
    else
    {
        std::cout << " no scorep.conf" << std::endl;
    } 

    this->toolBar = new ScorePionToolBar( service );
    service->addToolBar( toolBar );
    toolBar->hide();
    
    widget_ = new ScorePionPluginWidget(service->getCube(),  toolBar, filter_factory ); 
    
    service->addTab(  cubepluginapi::SYSTEM, this, cubepluginapi::OTHER_PLUGIN_TAB  );

    connect(toolBar, SIGNAL( addedMetric()), widget_, SLOT(addedMetric())  );
    
    connect ( toolBar, SIGNAL( changedCountersNum(int)), widget_, SLOT(adjustCountersNum( int )) );
    
    
    connect( service, SIGNAL( contextMenuIsShown( cubepluginapi::TreeType, cubepluginapi::TreeItem* ) ),
        this, SLOT( contextMenuIsShown( cubepluginapi::TreeType, cubepluginapi::TreeItem* ) ) );
        return true;
}

void
ScorePionPlugin::cubeClosed()
{
    delete widget_;
    delete filter_factory;
}

/******************************************************************************************************
* start of TabInterface implementation
******************************************************************************************************/

/**
 * returns the tab label of the created system tab
 */
QString
ScorePionPlugin::label() const
{
    return "ScorePion";
}

/** returns the icon which will be placed left to the tab label */
QIcon
ScorePionPlugin::icon() const
{
    return QIcon( ":/scorepionicons/icon.png" );
}

/**
 * returns widget that will be placed into the tab
 */
QWidget*
ScorePionPlugin::widget()
{
    return widget_;
}


/** Returns the minimal width that is necessary to display all informations.
    This value is used in the menu item "Display->Optimize width".
 */
QSize
ScorePionPlugin::sizeHint() const
{
    return QSize( 100, 100 );
}

// ******************************************************************************************************


/******************************************************************************************************
* implementation of slots to handle signals of PluginServices
******************************************************************************************************/

/**
 * @brief ScorePionPlugin::loadSettings loads settings from global setting file
 * the plugin should create a group with its name as group label
 */
void
ScorePionPlugin::loadGlobalSettings( QSettings& settings )
{
    int numCalls = settings.value( "ScorePionPluginCalls", 0 ).toInt();

    service->debug() << "loadSettings: " << numCalls << endl;
}

/**
   Saves settings system specific file, for UNIX systems to $HOME/.config/FZJ/Cube.conf
 */
void
ScorePionPlugin::saveGlobalSettings( QSettings& settings )
{
    int numCalls = settings.value( "ScorePionPluginCalls", 0 ).toInt(); // 0 is default, if no value exists
    settings.setValue( "ScorePionPluginCalls", ++numCalls );
}

QString
ScorePionPlugin::settingName()
{
    return "ScorePionPlugin";
}

void
ScorePionPlugin::contextMenuIsShown( cubepluginapi::TreeType type, cubepluginapi::TreeItem* item )
{
     
    if (type == cubepluginapi::CALLTREE || type == cubepluginapi::CALLFLAT)
    {
        contextItem     = item;
        contextTreeType = type;

        QAction*    excludeRegionNameToFilter = service->addContextMenuItem( type, " Exclude Region Name from Measurement" );
        connect( excludeRegionNameToFilter, SIGNAL( triggered() ), this, SLOT( exclRegNameToFilter() ) );
        
        QAction*    excludeRegionFileToFilter = service->addContextMenuItem( type, " Exclude Region File from Measurement" );
        connect( excludeRegionFileToFilter, SIGNAL( triggered() ), this, SLOT( exclRegFileToFilter() ) );

        QAction*    includeRegionNameToFilter = service->addContextMenuItem( type, " Include Region Name to Measurement" );
        connect( includeRegionNameToFilter, SIGNAL( triggered() ), this, SLOT( inclRegNameToFilter() ) );
        
        QAction*    includeRegionFileToFilter = service->addContextMenuItem( type, " Include Region File to Measurement" );
        connect( includeRegionFileToFilter, SIGNAL( triggered() ), this, SLOT( inclRegFileToFilter() ) );
        
        
        service->setMessage( "Update your Score-P Filter file..." ); // write the message to status bar
    }
}


void
ScorePionPlugin::inclRegNameToFilter()
{   
    bool over_selected = false;
    /** Returns selected items in the given tree  @param type METRICTREE, CALLTREE, CALLFLAT, SYSTEMTREE */
    const QList<cubepluginapi::TreeItem*>& selected = service->getSelections( service->getActiveTreeType( cubepluginapi::CALL ) );
    
    std::vector<cube::Vertex * > v;
    foreach ( cubepluginapi::TreeItem* item, selected)
    {
        if (item == contextItem)
            over_selected = true;        
        v.push_back( item->getCubeObject());
    }
    if (contextItem != NULL && !over_selected )
    {
        widget_->addIncludeRegion( contextItem->getCubeObject() );
    }
    else
    {
        widget_->addIncludeRegion( v );
    } 
}

void
ScorePionPlugin::inclRegFileToFilter()
{  
    bool over_selected = false;
    /** Returns selected items in the given tree  @param type METRICTREE, CALLTREE, CALLFLAT, SYSTEMTREE */
    const QList<cubepluginapi::TreeItem*>& selected = service->getSelections( service->getActiveTreeType( cubepluginapi::CALL ) );
    
    std::vector<cube::Vertex * > v;
    foreach ( cubepluginapi::TreeItem* item, selected)
    {
        if (item == contextItem)
            over_selected = true;        
        v.push_back( item->getCubeObject());
    }
    if (contextItem != NULL && !over_selected )
    {
        widget_->addIncludeFile( contextItem->getCubeObject() );    
    }
    else
    {
        widget_->addIncludeFile( v );
    } 
}


void
ScorePionPlugin::exclRegNameToFilter()
{
    bool over_selected = false;
    /** Returns selected items in the given tree  @param type METRICTREE, CALLTREE, CALLFLAT, SYSTEMTREE */
    const QList<cubepluginapi::TreeItem*>& selected = service->getSelections( service->getActiveTreeType( cubepluginapi::CALL ) );
    
    std::vector<cube::Vertex * > v;
    foreach ( cubepluginapi::TreeItem* item, selected)
    {
        if (item == contextItem)
            over_selected = true;        
        v.push_back( item->getCubeObject());
    }
    if (contextItem != NULL && !over_selected )
    {
        widget_->addExcludeRegion( contextItem->getCubeObject() );
    }
    else
    {
        widget_->addExcludeRegion( v );
    } 
}

void
ScorePionPlugin::exclRegFileToFilter()
{
    bool over_selected = false;
    /** Returns selected items in the given tree  @param type METRICTREE, CALLTREE, CALLFLAT, SYSTEMTREE */
    const QList<cubepluginapi::TreeItem*>& selected = service->getSelections( service->getActiveTreeType( cubepluginapi::CALL ) );
    
    std::vector<cube::Vertex * > v;
    foreach ( cubepluginapi::TreeItem* item, selected)
    {
        if (item == contextItem)
            over_selected = true;        
        v.push_back( item->getCubeObject());
    }
    if (contextItem != NULL && !over_selected )
    {
        widget_->addExcludeFile( contextItem->getCubeObject() );
    }
    else
    {
        widget_->addExcludeFile( v );
    } 
}




/**
 * @brief DemoPlugin::setActive is called when the tab gets activated or deactivated by selecting another tab
 * The tab related demo elements should only react on signals, if the tab is active. For that reason the
 * signals are disconnected if another tab becomes active.
 */
void
ScorePionPlugin::setActive( bool active )
{
    if ( active )
    {
        toolBar->showSelf();
//         connect( service, SIGNAL( contextMenuIsShown( cubepluginapi::TreeType, cubepluginapi::TreeItem* ) ),
//         this, SLOT( contextMenuIsShown( cubepluginapi::TreeType, cubepluginapi::TreeItem* ) ) );
    }
    else
    {
        toolBar->hideSelf();
//         service->disconnect(  service, SIGNAL( contextMenuIsShown( cubepluginapi::TreeType, cubepluginapi::TreeItem* ) ) ) ;
    }
}



void
ScorePionPlugin::valuesChanged()
{
    widget_->updateMeasurementLabels( );
}
